﻿//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                                          //
//   888888888888      888888888       88888888888  888888    888888            88888                       8888888      888888             //
//   88888   88888   8888888888888    888888888888  888888   888888             88888     888888888888      8888888      888888             //
//   88888   88888  88888     88888  888888   8888  888888  888888      8888888888888   888888    888888    8888888      888888             //
//   88888   88888  88888     88888  888888         8888888888888     888888888888888  888888      888888   8888888      888888             //
//   88888  88888   888888888888888  888888   8888  888888  888888    8888      88888  888888      888888   8888888      888888    		    //          
//   88888    8888  88888     88888  8888888888888  888888   888888   888888888888888   888888    888888    88888888888  8888888888888888   //
//   88888    8888  88888     88888   888888888888  888888    888888  888888888888888    88888888888888     88888888888  8888888888888888   //
//																																			//
//                                                                                                                       rackdoll.nl/blog   //
//                                                                                                                                			//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                                          //
//    Project: Konami Code Interpreter                                                                                                      //
//    Author : Rackdoll                                                                                                                     //
//    Package: nl.rackdoll.konami                                                                                                           //
//                                                                                                                                          //
//    USAGE:                                                                                                                                //
//    KCI.help();                                                                                                                           //
//    KCI.supportedCodes();                                                                                                                 //
//                                                                                                                                          //
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



package nl.rackdoll.konami {
	import flash.display.Stage;
	import flash.events.Event;
	import flash.events.EventDispatcher;
	import flash.events.KeyboardEvent;
	import flash.events.TimerEvent;
	import flash.utils.Timer;
	
	public class KCI extends EventDispatcher {
		public static const _KONAMI_CODE:String = 'UUDDLRLRba';
		public function KCI( $stage:Stage,$autoEmbedKonamiCode:Boolean = true, $timeFrame:Number = 1000.0, $codes:Vector.<String> = null ) {
			_timeFrame = $timeFrame;
			_codes     = ($codes)?$codes:_codes = new Vector.<String>();
			if ( $autoEmbedKonamiCode ) addCode( _KONAMI_CODE );
			_timer = new Timer( _timeFrame, 1 );
			_timer.addEventListener(TimerEvent.TIMER_COMPLETE, reset );
			$stage.addEventListener(KeyboardEvent.KEY_UP, onKeyUp );
		}
		public function addCode( $code:String ):void {
			_codes.push( $code );
		}
		public function removeCode( $code:String ):void {
			for (var i:int = 0; i < _codes.length; i++) {
				if ( $code === _codes[i] ) _codes.splice( i, 1 );
			}
		}
		
		public function setTimeFrame( $timeFrame:Number ):void {
			timeFrame = $timeFrame;
		}
		public function set timeFrame( $timeFrame:Number ):void {
			_timeFrame = $timeFrame;
			reset();
			_timer.delay = _timeFrame;
		}
		public function get timeFrame():Number {
			return _timeFrame;
		}
		
		
		public static function help($traceOut:Boolean = true):String {
			var help:String = "\n\n***************************************************************************\nKonami Code Interpreter.\nRackdollLabs@2009\nMakes your flash application Konami Code enabled.\n***************************************************************************\n\nTo make your flash application Code enabled, here's what you have to do.\n\nFirst make a new instance of the Konami Code Interpreter:\n-----------------------------------------\nvar k:KCI = new KCI( yourStageObject );\n\n\nSecond, you have to add a listener to the instance:\n-----------------------------------------\nk.addEventListener( KCI._KONAMI_CODE, yourListenerFunction );\n\n\nThird, create the listener function:\n-----------------------------------------\nprivate function yourListenerFunction (e:Event ):void {\n\ttrace( e.type + ' MATCHED!' )\n};\n\n\n\nOPTIONAL FUNCTIONS:\n-----------------------------------------\n\n!!TIP!!\nUse KCI.help() and KCI.supportedCodes()\nto help you on your way!!\n\nThe KCI auto includes the Konami Code (*optional)\n( up,up,down,down,left,right,left,right,b,a)\nbut you can also add your own code.\n\nAdding your own code:\n-----------------------------------------\nk.addCode('yourString');\nk.addEventListener( 'yourString',yourListenerFunction);\n\n\nChanging the input time for a code:\n-----------------------------------------\nk.setTimeFrame( $newTimeFrameInMilliSeconds );"+supportedCodes(false)+"\n\n\nThats all!\nHave fun!\nRackdoll";
			if( $traceOut ) trace( help );
			return help;
		}
		public static function supportedCodes($traceOut:Boolean = true):String {
			if ( $traceOut ) trace( CodeConverter.getSupportedCodes() );
			return CodeConverter.getSupportedCodes();
		}
		
		private function reset(e:Event = null):void {
			_codeCount = 0;
			_codeString = "";
			_timer.reset();
		}
		private function onKeyUp( e:KeyboardEvent ):void {
			_codeString += CodeConverter.convert(e.keyCode );
			if ( checkCodeSegment( _codeString, _codeCount ) ) {
				_codeCount++;
				checkTimer();
				checkFullCode( _codeString )
			}else {
				reset();
			}
		}
		private function checkCodeSegment( $code:String, $segment:uint = 0):Boolean {
			for (var i:int = 0; i < _codes.length; i++) {
				var toCheck:String = _codes[i].substr(0, $segment + 1 );
				if ( toCheck.match($code) ) return true;
			}
			return false;
		}
		private function checkFullCode( $code:String ):void {
			for (var i:int = 0; i < _codes.length; i++) {
				if ( $code === _codes[i] ) {
					dispatchEvent( new Event( _codeString, true ) );
					reset();
				}
			}
		}
		private function checkTimer():void {
			if (!_timer.running ){
				_timer.start();
			}else{
				_timer.reset();
				_timer.start();
			}
		}
		
		private var _codes:Vector.<String>;
		private var _timeFrame:Number;
		private var _timer:Timer;
		private var _codeCount:uint = 0;
		private var _codeString:String = "";
	}
}
internal class CodeConverter {
	public static function convert($code:uint):String {
		var _hashTable:String = ($code<186)?($code<41)?'LURD':'abcdefghijklmnopqrstuvwxyz':($code<219)?';+,-./`':"[|]'";
		var _result   :String = _hashTable.charAt(($code<100)?($code<41)? $code-37:$code-65:($code<219)?$code-186:$code-219);
		return (_result!="")?_result:$code.toString();
	}
	public static function getSupportedCodes():String {
		return "\n\n\n**********************************************\nSUPPORTED CODES\n**********************************************\n\nALPHABET : " + 'a b c d e f g h i j k l m n o p q r s t u v w x y z\n-------------------------\n\n' + "SPECIAL CHARACTERS: " + ' ; + , - . / ` ' + "[ | ] '\n-------------------------\n\n" + "KEY CONTROLS:\n-------------------------\n" +"\nLEFT: L\nUP: U\nRIGHT: R\nDOWN: D\n"; 
	}
}